@@ -1,25 +1,23 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1999-2001
+ * Copyright (c) 1999-2004
  *	Sleepycat Software.  All rights reserved.
+ *
+ * $Id: qam_conv.c,v 11.17 2004/01/28 03:36:19 bostic Exp $
  */
 
 #include "db_config.h"
 
-#ifndef lint
-static const char revid[] = "$Id: qam_conv.c,v 11.7 2001/01/25 18:23:01 bostic Exp $";
-#endif /* not lint */
-
 #ifndef NO_SYSTEM_INCLUDES
 #include <sys/types.h>
 #endif
 
 #include "db_int.h"
-#include "db_page.h"
-#include "qam.h"
-#include "db_swap.h"
-#include "db_am.h"
+#include "dbinc/db_page.h"
+#include "dbinc/db_swap.h"
+#include "dbinc/db_am.h"
+#include "dbinc/qam.h"
 
 /*
  * __qam_mswap --
@@ -43,6 +41,8 @@ __qam_mswap(pg)
 	SWAP32(p);		/* re_pad */
 	SWAP32(p);		/* rec_page */
 	SWAP32(p);		/* page_ext */
+	p += 91 * sizeof(u_int32_t); /* unused */
+	SWAP32(p);		/* crypto_magic */
 
 	return (0);
 }
@@ -68,7 +68,7 @@ __qam_pgin_out(dbenv, pg, pp, cookie)
 	COMPQUIET(pg, 0);
 	COMPQUIET(dbenv, NULL);
 	pginfo = (DB_PGINFO *)cookie->data;
-	if (!pginfo->needswap)
+	if (!F_ISSET(pginfo, DB_AM_SWAP))
 		return (0);
 
 	h = pp;
